@@ -30,6 +30,11 @@
 #include <time.h>
 #include <string.h>
 #include <pthread.h>
+#ifdef HAVE_PTHREAD_NP_H
+# include <pthread_np.h>
+#endif
+
+#include <event2/event.h>
 
 #include "conffile.h"
 #include "logger.h"
@@ -661,13 +666,8 @@ cache_daap_reply_add(const char *query, struct evbuffer *evbuf)
   size_t datalen;
   int ret;
 
-#ifdef HAVE_LIBEVENT2
   datalen = evbuffer_get_length(evbuf);
   data = evbuffer_pullup(evbuf, -1);
-#else
-  datalen = EVBUFFER_LENGTH(evbuf);
-  data = EVBUFFER_DATA(evbuf);
-#endif
 
   ret = sqlite3_prepare_v2(g_db_hdl, Q_TMPL, -1, &stmt, 0);
   if (ret != SQLITE_OK)
@@ -1053,6 +1053,8 @@ cache_artwork_delete_by_path_impl(struct cache_command *cmd)
       return -1;
     }
 
+  DPRINTF(E_DBG, L_CACHE, "Deleted %d rows\n", sqlite3_changes(g_db_hdl));
+
   return 0;
 
 #undef Q_TMPL_DEL
@@ -1123,13 +1125,8 @@ cache_artwork_add_impl(struct cache_command *cmd)
       return -1;
     }
 
-#ifdef HAVE_LIBEVENT2
   datalen = evbuffer_get_length(cmd->arg.evbuf);
   data = evbuffer_pullup(cmd->arg.evbuf, -1);
-#else
-  datalen = EVBUFFER_LENGTH(cmd->arg.evbuf);
-  data = EVBUFFER_DATA(cmd->arg.evbuf);
-#endif
 
   sqlite3_bind_int64(stmt, 1, cmd->arg.persistentid);
   sqlite3_bind_int(stmt, 2, cmd->arg.max_w);
@@ -1223,6 +1220,7 @@ cache_artwork_get_impl(struct cache_command *cmd)
   if (!cmd->arg.evbuf)
     {
       DPRINTF(E_LOG, L_CACHE, "Error: Artwork evbuffer is NULL\n");
+      ret = -1;
       goto error_get;
     }
 
@@ -1230,6 +1228,7 @@ cache_artwork_get_impl(struct cache_command *cmd)
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_CACHE, "Out of memory for artwork evbuffer\n");
+      ret = -1;
       goto error_get;
     }
 
@@ -1249,7 +1248,7 @@ cache_artwork_get_impl(struct cache_command *cmd)
   sqlite3_finalize(stmt);
   sqlite3_free(query);
 
-  return -1;
+  return ret;
 #undef Q_TMPL
 }
 
@@ -1685,7 +1684,11 @@ cache_artwork_get(int type, int64_t persistentid, int max_w, int max_h, int *cac
   int ret;
 
   if (!g_initialized)
-    return -1;
+    {
+      *cached = 0;
+      *format = 0;
+      return 0;
+    }
 
   command_init(&cmd);
 
@@ -1793,22 +1796,22 @@ cache_init(void)
       return 0;
     }
 
-# if defined(__linux__)
+#ifdef HAVE_PIPE2
   ret = pipe2(g_exit_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(g_exit_pipe);
-# endif
+#endif
   if (ret < 0)
     {
-      DPRINTF(E_LOG, L_CACHE, "Could not create pipe: %s\n", strerror(errno));
+      DPRINTF(E_LOG, L_CACHE, "Could not create exit pipe: %s\n", strerror(errno));
       goto exit_fail;
     }
 
-# if defined(__linux__)
+#ifdef HAVE_PIPE2
   ret = pipe2(g_cmd_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(g_cmd_pipe);
-# endif
+#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_CACHE, "Could not create command pipe: %s\n", strerror(errno));
@@ -1822,7 +1825,6 @@ cache_init(void)
       goto evbase_fail;
     }
 
-#ifdef HAVE_LIBEVENT2
   g_exitev = event_new(evbase_cache, g_exit_pipe[0], EV_READ, exit_cb, NULL);
   if (!g_exitev)
     {
@@ -1843,34 +1845,6 @@ cache_init(void)
       DPRINTF(E_LOG, L_CACHE, "Could not create cache event\n");
       goto evnew_fail;
     }
-#else
-  g_exitev = (struct event *)malloc(sizeof(struct event));
-  if (!g_exitev)
-    {
-      DPRINTF(E_LOG, L_CACHE, "Could not create exit event\n");
-      goto evnew_fail;
-    }
-  event_set(g_exitev, g_exit_pipe[0], EV_READ, exit_cb, NULL);
-  event_base_set(evbase_cache, g_exitev);
-
-  g_cmdev = (struct event *)malloc(sizeof(struct event));
-  if (!g_cmdev)
-    {
-      DPRINTF(E_LOG, L_CACHE, "Could not create cmd event\n");
-      goto evnew_fail;
-    }
-  event_set(g_cmdev, g_cmd_pipe[0], EV_READ, command_cb, NULL);
-  event_base_set(evbase_cache, g_cmdev);
-
-  g_cacheev = (struct event *)malloc(sizeof(struct event));
-  if (!g_cacheev)
-    {
-      DPRINTF(E_LOG, L_CACHE, "Could not create cache event\n");
-      goto evnew_fail;
-    }
-  event_set(g_cacheev, -1, EV_TIMEOUT, cache_daap_update_cb, NULL);
-  event_base_set(evbase_cache, g_cacheev);
-#endif
 
   event_add(g_exitev, NULL);
   event_add(g_cmdev, NULL);
@@ -1885,6 +1859,12 @@ cache_init(void)
       goto thread_fail;
     }
 
+#if defined(HAVE_PTHREAD_SETNAME_NP)
+  pthread_setname_np(tid_cache, "cache");
+#elif defined(HAVE_PTHREAD_SET_NAME_NP)
+  pthread_set_name_np(tid_cache, "cache");
+#endif
+
   return 0;
   
  thread_fail:
