@@ -37,11 +37,7 @@
 
 #include <plist/plist.h>
 
-#ifdef HAVE_LIBEVENT2
-# include <event2/http.h>
-#else
-# include "evhttp/evhttp.h"
-#endif
+#include <event2/http.h>
 
 #include "logger.h"
 #include "db.h"
