@@ -29,7 +29,7 @@
 #include <sys/stat.h>
 #include <pthread.h>
 
-#include <event.h>
+#include <event2/event.h>
 
 #include <libavutil/log.h>
 
@@ -43,7 +43,7 @@ static int threshold;
 static int console;
 static char *logfilename;
 static FILE *logfile;
-static char *labels[] = { "config", "daap", "db", "httpd", "http", "main", "mdns", "misc", "rsp", "scan", "xcode", "event", "remote", "dacp", "ffmpeg", "artwork", "player", "raop", "laudio", "dmap", "dbperf", "spotify", "lastfm", "cache", "mpd" };
+static char *labels[] = { "config", "daap", "db", "httpd", "http", "main", "mdns", "misc", "rsp", "scan", "xcode", "event", "remote", "dacp", "ffmpeg", "artwork", "player", "raop", "laudio", "dmap", "dbperf", "spotify", "lastfm", "cache", "mpd", "stream", "cast" };
 static char *severities[] = { "FATAL", "LOG", "WARN", "INFO", "DEBUG", "SPAM" };
 
 
@@ -160,19 +160,19 @@ logger_libevent(int severity, const char *msg)
 {
   switch (severity)
     {
-      case _EVENT_LOG_DEBUG:
+      case EVENT_LOG_DEBUG:
 	severity = E_DBG;
 	break;
 
-      case _EVENT_LOG_ERR:
+      case EVENT_LOG_ERR:
 	severity = E_LOG;
 	break;
 
-      case _EVENT_LOG_WARN:
+      case EVENT_LOG_WARN:
 	severity = E_WARN;
 	break;
 
-      case _EVENT_LOG_MSG:
+      case EVENT_LOG_MSG:
 	severity = E_INFO;
 	break;
 
