@@ -35,7 +35,11 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/queue.h>
+#include <time.h>
 #include <pthread.h>
+#ifdef HAVE_PTHREAD_NP_H
+# include <pthread_np.h>
+#endif
 
 #include <dlfcn.h>
 #include <libspotify/api.h>
@@ -44,10 +48,13 @@
 #include "logger.h"
 #include "conffile.h"
 #include "filescanner.h"
+#include "cache.h"
 
 
 /* How long to wait for audio (in sec) before giving up */
 #define SPOTIFY_TIMEOUT 20
+/* How long to wait for artwork (in sec) before giving up */
+#define SPOTIFY_ARTWORK_TIMEOUT 3
 
 /* --- Types --- */
 typedef struct audio_fifo_data
@@ -88,6 +95,11 @@ struct artwork_get_param
   char *path;
   int max_w;
   int max_h;
+
+  sp_image *image;
+  pthread_mutex_t mutex;
+  pthread_cond_t cond;
+  int is_loaded;
 };
 
 struct spotify_command;
@@ -206,6 +218,7 @@ typedef int          (*fptr_sp_playlist_num_tracks_t)(sp_playlist *playlist);
 typedef sp_track*    (*fptr_sp_playlist_track_t)(sp_playlist *playlist, int index);
 typedef bool         (*fptr_sp_playlist_is_loaded_t)(sp_playlist *playlist);
 typedef int          (*fptr_sp_playlist_track_create_time_t)(sp_playlist *playlist, int index);
+typedef sp_user*     (*fptr_sp_playlist_owner_t)(sp_playlist *playlist);
 
 typedef sp_error     (*fptr_sp_track_error_t)(sp_track *track);
 typedef bool         (*fptr_sp_track_is_loaded_t)(sp_track *track);
@@ -238,6 +251,11 @@ typedef sp_error     (*fptr_sp_image_error_t)(sp_image *image);
 typedef sp_imageformat (*fptr_sp_image_format_t)(sp_image *image);
 typedef const void*  (*fptr_sp_image_data_t)(sp_image *image, size_t *data_size);
 typedef sp_error     (*fptr_sp_image_release_t)(sp_image *image);
+typedef sp_error     (*fptr_sp_image_add_load_callback_t)(sp_image *image, image_loaded_cb *callback, void *userdata);
+typedef sp_error     (*fptr_sp_image_remove_load_callback_t)(sp_image *image, image_loaded_cb *callback, void *userdata);
+
+typedef const char*  (*fptr_sp_user_display_name_t)(sp_user *user);
+typedef const char*  (*fptr_sp_user_canonical_name_t)(sp_user *user);
 
 /* Define actual function pointers */
 fptr_sp_error_message_t fptr_sp_error_message;
@@ -268,6 +286,7 @@ fptr_sp_playlist_num_tracks_t fptr_sp_playlist_num_tracks;
 fptr_sp_playlist_track_t fptr_sp_playlist_track;
 fptr_sp_playlist_is_loaded_t fptr_sp_playlist_is_loaded;
 fptr_sp_playlist_track_create_time_t fptr_sp_playlist_track_create_time;
+fptr_sp_playlist_owner_t fptr_sp_playlist_owner;
 
 fptr_sp_track_error_t fptr_sp_track_error;
 fptr_sp_track_is_loaded_t fptr_sp_track_is_loaded;
@@ -300,6 +319,11 @@ fptr_sp_image_error_t fptr_sp_image_error;
 fptr_sp_image_format_t fptr_sp_image_format;
 fptr_sp_image_data_t fptr_sp_image_data;
 fptr_sp_image_release_t fptr_sp_image_release;
+fptr_sp_image_add_load_callback_t fptr_sp_image_add_load_callback;
+fptr_sp_image_remove_load_callback_t fptr_sp_image_remove_load_callback;
+
+fptr_sp_user_display_name_t fptr_sp_user_display_name;
+fptr_sp_user_canonical_name_t fptr_sp_user_canonical_name;
 
 /* Assign function pointers to libspotify symbol */
 static int
@@ -337,6 +361,7 @@ fptr_assign_all()
    && (fptr_sp_playlist_track = dlsym(h, "sp_playlist_track"))
    && (fptr_sp_playlist_is_loaded = dlsym(h, "sp_playlist_is_loaded"))
    && (fptr_sp_playlist_track_create_time = dlsym(h, "sp_playlist_track_create_time"))
+   && (fptr_sp_playlist_owner = dlsym(h, "sp_playlist_owner"))
    && (fptr_sp_track_error = dlsym(h, "sp_track_error"))
    && (fptr_sp_track_is_loaded = dlsym(h, "sp_track_is_loaded"))
    && (fptr_sp_track_name = dlsym(h, "sp_track_name"))
@@ -364,6 +389,10 @@ fptr_assign_all()
    && (fptr_sp_image_format = dlsym(h, "sp_image_format"))
    && (fptr_sp_image_data = dlsym(h, "sp_image_data"))
    && (fptr_sp_image_release = dlsym(h, "sp_image_release"))
+   && (fptr_sp_image_add_load_callback = dlsym(h, "sp_image_add_load_callback"))
+   && (fptr_sp_image_remove_load_callback = dlsym(h, "sp_image_remove_load_callback"))
+   && (fptr_sp_user_display_name = dlsym(h, "sp_user_display_name"))
+   && (fptr_sp_user_canonical_name = dlsym(h, "sp_user_canonical_name"))
    ;
 
   err = dlerror();
@@ -474,9 +503,7 @@ spotify_metadata_get(sp_track *track, struct media_file_info *mfi, const char *p
 {
   cfg_t *spotify_cfg;
   bool artist_override;
-  bool starred_artist_override;
   bool album_override;
-  bool starred_album_override;
   sp_album *album;
   sp_artist *artist;
   sp_albumtype albumtype;
@@ -486,9 +513,7 @@ spotify_metadata_get(sp_track *track, struct media_file_info *mfi, const char *p
 
   spotify_cfg = cfg_getsec(cfg, "spotify");
   artist_override = cfg_getbool(spotify_cfg, "artist_override");
-  starred_artist_override = cfg_getbool(spotify_cfg, "starred_artist_override");
   album_override = cfg_getbool(spotify_cfg, "album_override");
-  starred_album_override = cfg_getbool(spotify_cfg, "starred_album_override");
 
   album = fptr_sp_track_album(track);
   if (!album)
@@ -508,11 +533,9 @@ spotify_metadata_get(sp_track *track, struct media_file_info *mfi, const char *p
    * - starred_artist_override in config is set to true and track is part of the starred playlist
    */
   compilation = ((albumtype == SP_ALBUMTYPE_COMPILATION)
-		  || (starred && starred_artist_override)
-		  || (!starred && artist_override));
+		  || artist_override);
 
-  if ((starred && starred_album_override)
-      || (!starred && album_override))
+  if (album_override)
     albumname = strdup(pltitle);
   else
     albumname = strdup(fptr_sp_album_name(album));
@@ -559,6 +582,9 @@ spotify_track_save(int plid, sp_track *track, const char *pltitle, int time_adde
   sp_link *link;
   char url[1024];
   int ret;
+  int dir_id;
+  char virtual_path[PATH_MAX];
+
 
   if (!fptr_sp_track_is_loaded(track))
     {
@@ -604,25 +630,89 @@ spotify_track_save(int plid, sp_track *track, const char *pltitle, int time_adde
       return -1;
     }
 
-  filescanner_process_media(url, time(NULL), 0, F_SCAN_TYPE_SPOTIFY, &mfi);
+  ret = snprintf(virtual_path, sizeof(virtual_path), "/spotify:/%s", mfi.artist);
+  if ((ret < 0) || (ret >= sizeof(virtual_path)))
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Virtual path exceeds PATH_MAX (/spotify:/%s)\n", mfi.artist);
+      free_mfi(&mfi, 1);
+      return -1;
+    }
+  dir_id = db_directory_addorupdate(virtual_path, 0, DIR_SPOTIFY);
+  if (dir_id <= 0)
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Could not add or update directory '%s'\n", virtual_path);
+      free_mfi(&mfi, 1);
+      return -1;
+    }
+  ret = snprintf(virtual_path, sizeof(virtual_path), "/spotify:/%s/%s", mfi.artist, mfi.album);
+  if ((ret < 0) || (ret >= sizeof(virtual_path)))
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Virtual path exceeds PATH_MAX (/spotify:/%s/%s)\n", mfi.artist, mfi.album);
+      free_mfi(&mfi, 1);
+      return -1;
+    }
+  dir_id = db_directory_addorupdate(virtual_path, 0, dir_id);
+  if (dir_id <= 0)
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Could not add or update directory '%s'\n", virtual_path);
+      free_mfi(&mfi, 1);
+      return -1;
+    }
+
+  filescanner_process_media(url, time(NULL), 0, F_SCAN_TYPE_SPOTIFY, &mfi, dir_id);
 
   free_mfi(&mfi, 1);
 
   return 0;
 }
 
+static int
+spotify_playlist_cleanupfiles()
+{
+  struct query_params qp;
+  char *path;
+  int ret;
+
+  memset(&qp, 0, sizeof(struct query_params));
+
+  qp.type = Q_BROWSE_PATH;
+  qp.sort = S_NONE;
+  qp.filter = "f.path LIKE 'spotify:%%' AND NOT f.path IN (SELECT filepath FROM playlistitems)";
+
+  ret = db_query_start(&qp);
+  if (ret < 0)
+    {
+      db_query_end(&qp);
+
+      return -1;
+    }
+
+  while (((ret = db_query_fetch_string(&qp, &path)) == 0) && (path))
+    {
+      cache_artwork_delete_by_path(path);
+    }
+
+  db_query_end(&qp);
+
+  db_spotify_files_delete();
+
+  return 0;
+}
+
 static int
 spotify_playlist_save(sp_playlist *pl)
 {
   struct playlist_info *pli;
   sp_track *track;
   sp_link *link;
+  sp_user *owner;
   char url[1024];
   const char *name;
+  const char *ownername;
   int plid;
   int num_tracks;
   char virtual_path[PATH_MAX];
-  int time;
+  int created;
   int ret;
   int i;
   
@@ -667,9 +757,23 @@ spotify_playlist_save(sp_playlist *pl)
     }
   fptr_sp_link_release(link);
 
-  pli = db_pl_fetch_bypath(url);
+  owner = fptr_sp_playlist_owner(pl);
+  if (owner)
+    {
+      DPRINTF(E_DBG, L_SPOTIFY, "Playlist '%s' owner: '%s' (canonical) / '%s' (display)\n",
+	  name, fptr_sp_user_canonical_name(owner), fptr_sp_user_display_name(owner));
 
-  snprintf(virtual_path, PATH_MAX, "/spotify:/%s", name);
+      ownername = fptr_sp_user_canonical_name(owner);
+
+      snprintf(virtual_path, PATH_MAX, "/spotify:/%s (%s)", name, ownername);
+    }
+  else
+    {
+      snprintf(virtual_path, PATH_MAX, "/spotify:/%s", name);
+    }
+
+
+  pli = db_pl_fetch_bypath(url);
 
   if (pli)
     {
@@ -681,6 +785,7 @@ spotify_playlist_save(sp_playlist *pl)
       pli->title = strdup(name);
       free(pli->virtual_path);
       pli->virtual_path = strdup(virtual_path);
+      pli->directory_id = DIR_SPOTIFY;
 
       ret = db_pl_update(pli);
       if (ret < 0)
@@ -712,6 +817,7 @@ spotify_playlist_save(sp_playlist *pl)
       pli->path = strdup(url);
       pli->virtual_path = strdup(virtual_path);
       pli->parent_id = g_base_plid;
+      pli->directory_id = DIR_SPOTIFY;
 
       ret = db_pl_add(pli, &plid);
       if ((ret < 0) || (plid < 1))
@@ -736,15 +842,17 @@ spotify_playlist_save(sp_playlist *pl)
 	  continue;
 	}
 
-      time = fptr_sp_playlist_track_create_time(pl, i);
+      created = fptr_sp_playlist_track_create_time(pl, i);
 
-      ret = spotify_track_save(plid, track, name, time);
+      ret = spotify_track_save(plid, track, name, created);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error saving track %d to playlist '%s' (id %d)\n", i, name, plid);
 	  continue;
 	}
     }
+
+  spotify_playlist_cleanupfiles();
   db_transaction_end();
 
   return plid;
@@ -860,6 +968,7 @@ playlist_removed(sp_playlistcontainer *pc, sp_playlist *pl, int position, void *
   free_pli(pli, 0);
 
   db_spotify_pl_delete(plid);
+  spotify_playlist_cleanupfiles();
 }
 
 /**
@@ -892,6 +1001,19 @@ static sp_playlistcontainer_callbacks pc_callbacks = {
 /* --------------------- INTERNAL PLAYBACK AND AUDIO ----------------------- */
 /*            Should only be called from within the spotify thread           */
 
+static void
+mk_reltime(struct timespec *ts, time_t sec)
+{
+#if _POSIX_TIMERS > 0
+  clock_gettime(CLOCK_REALTIME, ts);
+#else
+  struct timeval tv;
+  gettimeofday(&tv, NULL);
+  TIMEVAL_TO_TIMESPEC(&tv, ts);
+#endif
+  ts->tv_sec += sec;
+}
+
 static void
 audio_fifo_flush(void)
 {
@@ -912,12 +1034,12 @@ audio_fifo_flush(void)
 }
 
 static int
-playback_play(struct spotify_command *cmd)
+playback_setup(struct spotify_command *cmd)
 {
   sp_track *track;
   sp_error err;
 
-  DPRINTF(E_DBG, L_SPOTIFY, "Starting playback\n");
+  DPRINTF(E_DBG, L_SPOTIFY, "Setting up for playback\n");
 
   if (SP_CONNECTION_STATE_LOGGED_IN != fptr_sp_session_connectionstate(g_sess))
     {
@@ -947,6 +1069,16 @@ playback_play(struct spotify_command *cmd)
 
   audio_fifo_flush();
 
+  return 0;
+}
+
+static int
+playback_play(struct spotify_command *cmd)
+{
+  sp_error err;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Starting playback\n");
+
   err = fptr_sp_session_player_play(g_sess, 1);
   if (SP_ERROR_OK != err)
     {
@@ -980,25 +1112,6 @@ playback_pause(struct spotify_command *cmd)
   return 0;
 }
 
-static int
-playback_resume(struct spotify_command *cmd)
-{
-  sp_error err;
-
-  DPRINTF(E_DBG, L_SPOTIFY, "Resuming playback\n");
-
-  err = fptr_sp_session_player_play(g_sess, 1);
-  if (SP_ERROR_OK != err)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Playback resume failed: %s\n", fptr_sp_error_message(err));
-      return -1;
-    }
-
-  g_state = SPOTIFY_STATE_PLAYING;
-
-  return 0;
-}
-
 static int
 playback_stop(struct spotify_command *cmd)
 {
@@ -1071,7 +1184,7 @@ audio_get(struct spotify_command *cmd)
 
   // If spotify was paused begin by resuming playback
   if (g_state == SPOTIFY_STATE_PAUSED)
-    playback_resume(NULL);
+    playback_play(NULL);
 
   pthread_mutex_lock(&g_audio_fifo->mutex);
 
@@ -1094,16 +1207,8 @@ audio_get(struct spotify_command *cmd)
 	       (timeout < SPOTIFY_TIMEOUT) )
 	{
 	  DPRINTF(E_DBG, L_SPOTIFY, "Waiting for audio\n");
-#if _POSIX_TIMERS > 0
-	  clock_gettime(CLOCK_REALTIME, &ts);
-#else
-	  struct timeval tv;
-	  gettimeofday(&tv, NULL);
-	  TIMEVAL_TO_TIMESPEC(&tv, &ts);
-#endif
-	  ts.tv_sec += 5;
 	  timeout += 5;
-
+	  mk_reltime(&ts, 5);
 	  pthread_cond_timedwait(&g_audio_fifo->cond, &g_audio_fifo->mutex, &ts);
 	}
 
@@ -1140,6 +1245,92 @@ audio_get(struct spotify_command *cmd)
   return processed;
 }
 
+static void
+artwork_loaded_cb(sp_image *image, void *userdata)
+{
+  struct spotify_command *cmd = userdata;
+
+  pthread_mutex_lock(&cmd->arg.artwork.mutex);
+
+  cmd->arg.artwork.is_loaded = 1;
+
+  pthread_cond_signal(&cmd->arg.artwork.cond);
+  pthread_mutex_unlock(&cmd->arg.artwork.mutex);
+}
+
+static int
+artwork_get_bh(struct spotify_command *cmd)
+{
+  sp_imageformat imageformat;
+  sp_error err;
+  const void *data;
+  size_t data_size;
+  int ret;
+
+  sp_image *image = cmd->arg.artwork.image;
+  char *path = cmd->arg.artwork.path;
+
+  if (!cmd->arg.artwork.is_loaded)
+    {
+      DPRINTF(E_DBG, L_SPOTIFY, "Request for artwork timed out: %s\n", path);
+
+      fptr_sp_image_remove_load_callback(image, artwork_loaded_cb, cmd);
+      goto fail;
+    }
+
+  err = fptr_sp_image_error(image);
+  if (err != SP_ERROR_OK)
+    {
+      DPRINTF(E_WARN, L_SPOTIFY, "Getting artwork (%s) failed, Spotify error: %s\n", path, fptr_sp_error_message(err));
+      goto fail;
+    }
+
+  if (!fptr_sp_image_is_loaded(image))
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Load callback returned, but no image? Possible bug: %s\n", path);
+      goto fail;
+    }
+
+  imageformat = fptr_sp_image_format(image);
+  if (imageformat != SP_IMAGE_FORMAT_JPEG)
+    {
+      DPRINTF(E_WARN, L_SPOTIFY, "Getting artwork failed, invalid image format from Spotify: %s\n", path);
+      goto fail;
+    }
+
+  data = fptr_sp_image_data(image, &data_size);
+  if (!data || (data_size == 0))
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Getting artwork failed, no image data from Spotify: %s\n", path);
+      goto fail;
+    }
+
+  ret = evbuffer_expand(cmd->arg.artwork.evbuf, data_size);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Out of memory for artwork\n");
+      goto fail;
+    }
+
+  ret = evbuffer_add(cmd->arg.artwork.evbuf, data, data_size);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SPOTIFY, "Could not add Spotify image to event buffer\n");
+      goto fail;
+    }
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Spotify artwork loaded ok\n");
+
+  fptr_sp_image_release(image);
+
+  return data_size;
+
+ fail:
+  fptr_sp_image_release(image);
+
+  return -1;
+}
+
 static int
 artwork_get(struct spotify_command *cmd)
 {
@@ -1150,11 +1341,7 @@ artwork_get(struct spotify_command *cmd)
   const byte *image_id;
   sp_image *image;
   sp_image_size image_size;
-  sp_imageformat imageformat;
   sp_error err;
-  const void *data;
-  size_t data_size;
-  int ret;
 
   path = cmd->arg.artwork.path;
 
@@ -1201,53 +1388,27 @@ artwork_get(struct spotify_command *cmd)
       goto level2_exit;
     }
 
-  // We want to be fast, so no waiting for the image to load
-  if (!fptr_sp_image_is_loaded(image))
-    goto level3_exit;
+  fptr_sp_link_release(link);
 
-  err = fptr_sp_image_error(image);
+  cmd->arg.artwork.image = image;
+
+  /* If the image is ready we can return it straight away, otherwise we will
+   * let the calling thread wait, since the Spotify thread should not wait
+   */
+  if ( (cmd->arg.artwork.is_loaded = fptr_sp_image_is_loaded(image)) )
+    return artwork_get_bh(cmd);
+
+  DPRINTF(E_SPAM, L_SPOTIFY, "Will wait for Spotify to call artwork_loaded_cb\n");
+
+  /* Async - we will return to spotify_artwork_get which will wait for callback */
+  err = fptr_sp_image_add_load_callback(image, artwork_loaded_cb, cmd);
   if (err != SP_ERROR_OK)
     {
-      DPRINTF(E_WARN, L_SPOTIFY, "Getting artwork failed, Spotify error: %s\n", fptr_sp_error_message(err));
-      goto level3_exit;
+      DPRINTF(E_WARN, L_SPOTIFY, "Adding artwork cb failed, Spotify error: %s\n", fptr_sp_error_message(err));
+      return -1;
     }
 
-  imageformat = fptr_sp_image_format(image);
-  if (imageformat != SP_IMAGE_FORMAT_JPEG)
-    {
-      DPRINTF(E_WARN, L_SPOTIFY, "Getting artwork failed, invalid image format from Spotify: %s\n", path);
-      goto level3_exit;
-    }
-
-  data = fptr_sp_image_data(image, &data_size);
-  if (!data || (data_size == 0))
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Getting artwork failed, no image data from Spotify: %s\n", path);
-      goto level3_exit;
-    }
-
-  ret = evbuffer_expand(cmd->arg.artwork.evbuf, data_size);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Out of memory for artwork\n");
-      goto level3_exit;
-    }
-
-  ret = evbuffer_add(cmd->arg.artwork.evbuf, data, data_size);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not add Spotify image to event buffer\n");
-      goto level3_exit;
-    }
-
-  DPRINTF(E_DBG, L_SPOTIFY, "Spotify artwork loaded ok\n");
-
-  fptr_sp_image_release(image);
-
-  return data_size;
-
- level3_exit:
-  fptr_sp_image_release(image);
+  return 0;
 
  level2_exit:
   fptr_sp_link_release(link);
@@ -1281,7 +1442,7 @@ logged_in(sp_session *sess, sp_error error)
 
   DPRINTF(E_LOG, L_SPOTIFY, "Login to Spotify succeeded. Reloading playlists.\n");
 
-  db_spotify_purge();
+  db_directory_enable_bypath("/spotify:");
 
   pl = fptr_sp_session_starred_create(sess);
   fptr_sp_playlist_add_callbacks(pl, &pl_callbacks, NULL);
@@ -1626,13 +1787,13 @@ notify_cb(int fd, short what, void *arg)
 
 /* Thread: player */
 int
-spotify_playback_play(struct media_file_info *mfi)
+spotify_playback_setup(struct media_file_info *mfi)
 {
   struct spotify_command cmd;
   sp_link *link;
   int ret;
 
-  DPRINTF(E_DBG, L_SPOTIFY, "Playback request\n");
+  DPRINTF(E_DBG, L_SPOTIFY, "Playback setup request\n");
 
   link = fptr_sp_link_create_from_string(mfi->path);
   if (!link)
@@ -1643,7 +1804,7 @@ spotify_playback_play(struct media_file_info *mfi)
 
   command_init(&cmd);
 
-  cmd.func = playback_play;
+  cmd.func = playback_setup;
   cmd.arg.link = link;
 
   ret = sync_command(&cmd);
@@ -1653,6 +1814,46 @@ spotify_playback_play(struct media_file_info *mfi)
   return ret;
 }
 
+int
+spotify_playback_play()
+{
+  struct spotify_command cmd;
+  int ret;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Playback request\n");
+
+  command_init(&cmd);
+
+  cmd.func = playback_play;
+  cmd.arg.noarg = NULL;
+
+  ret = sync_command(&cmd);
+
+  command_deinit(&cmd);
+
+  return ret;
+}
+
+int
+spotify_playback_pause()
+{
+  struct spotify_command cmd;
+  int ret;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Pause request\n");
+
+  command_init(&cmd);
+
+  cmd.func = playback_pause;
+  cmd.arg.noarg = NULL;
+
+  ret = sync_command(&cmd);
+
+  command_deinit(&cmd);
+
+  return ret;
+}
+
 /* Thread: libspotify */
 void
 spotify_playback_pause_nonblock(void)
@@ -1678,27 +1879,6 @@ spotify_playback_pause_nonblock(void)
   nonblock_command(cmd);
 }
 
-/* Not used */
-int
-spotify_playback_resume(void)
-{
-  struct spotify_command cmd;
-  int ret;
-
-  DPRINTF(E_DBG, L_SPOTIFY, "Resume request\n");
-
-  command_init(&cmd);
-
-  cmd.func = playback_resume;
-  cmd.arg.noarg = NULL;
-
-  ret = sync_command(&cmd);
-
-  command_deinit(&cmd);
-
-  return ret;
-}
-
 /* Thread: player and libspotify */
 int
 spotify_playback_stop(void)
@@ -1787,11 +1967,12 @@ spotify_audio_get(struct evbuffer *evbuf, int wanted)
   return ret;
 }
 
-/* Thread: httpd (artwork) */
+/* Thread: httpd (artwork) and worker */
 int
 spotify_artwork_get(struct evbuffer *evbuf, char *path, int max_w, int max_h)
 {
   struct spotify_command cmd;
+  struct timespec ts;
   int ret;
 
   command_init(&cmd);
@@ -1802,8 +1983,24 @@ spotify_artwork_get(struct evbuffer *evbuf, char *path, int max_w, int max_h)
   cmd.arg.artwork.max_w = max_w;
   cmd.arg.artwork.max_h = max_h;
 
+  pthread_mutex_init(&cmd.arg.artwork.mutex, NULL);
+  pthread_cond_init(&cmd.arg.artwork.cond, NULL);
+
   ret = sync_command(&cmd);
 
+  // Artwork was not ready, wait for callback from libspotify
+  if (ret == 0)
+    {
+      pthread_mutex_lock(&cmd.arg.artwork.mutex);
+      mk_reltime(&ts, SPOTIFY_ARTWORK_TIMEOUT);
+      if (!cmd.arg.artwork.is_loaded)
+	pthread_cond_timedwait(&cmd.arg.artwork.cond, &cmd.arg.artwork.mutex, &ts);
+      pthread_mutex_unlock(&cmd.arg.artwork.mutex);
+
+      cmd.func = artwork_get_bh;
+      ret = sync_command(&cmd);
+    }
+
   command_deinit(&cmd);
 
   return ret;
@@ -1920,6 +2117,8 @@ spotify_login(char *path)
   char *password;
   int ret;
 
+  db_spotify_purge();
+
   if (!g_sess)
     {
       if (!g_libhandle)
@@ -1994,33 +2193,33 @@ spotify_init(void)
   if (ret < 0)
     goto assign_fail;
 
-# if defined(__linux__)
+#ifdef HAVE_PIPE2
   ret = pipe2(g_exit_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(g_exit_pipe);
-# endif
+#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create pipe: %s\n", strerror(errno));
       goto exit_fail;
     }
 
-# if defined(__linux__)
+#ifdef HAVE_PIPE2
   ret = pipe2(g_cmd_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(g_cmd_pipe);
-# endif
+#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create command pipe: %s\n", strerror(errno));
       goto cmd_fail;
     }
 
-# if defined(__linux__)
+#ifdef HAVE_PIPE2
   ret = pipe2(g_notify_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(g_notify_pipe);
-# endif
+#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not notify command pipe: %s\n", strerror(errno));
@@ -2034,7 +2233,6 @@ spotify_init(void)
       goto evbase_fail;
     }
 
-#ifdef HAVE_LIBEVENT2
   g_exitev = event_new(evbase_spotify, g_exit_pipe[0], EV_READ, exit_cb, NULL);
   if (!g_exitev)
     {
@@ -2055,34 +2253,6 @@ spotify_init(void)
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create notify event\n");
       goto evnew_fail;
     }
-#else
-  g_exitev = (struct event *)malloc(sizeof(struct event));
-  if (!g_exitev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create exit event\n");
-      goto evnew_fail;
-    }
-  event_set(g_exitev, g_exit_pipe[0], EV_READ, exit_cb, NULL);
-  event_base_set(evbase_spotify, g_exitev);
-
-  g_cmdev = (struct event *)malloc(sizeof(struct event));
-  if (!g_cmdev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create cmd event\n");
-      goto evnew_fail;
-    }
-  event_set(g_cmdev, g_cmd_pipe[0], EV_READ, command_cb, NULL);
-  event_base_set(evbase_spotify, g_cmdev);
-
-  g_notifyev = (struct event *)malloc(sizeof(struct event));
-  if (!g_notifyev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create notify event\n");
-      goto evnew_fail;
-    }
-  event_set(g_notifyev, g_notify_pipe[0], EV_READ | EV_TIMEOUT, notify_cb, NULL);
-  event_base_set(evbase_spotify, g_notifyev);
-#endif
 
   event_add(g_exitev, NULL);
   event_add(g_cmdev, NULL);
@@ -2141,6 +2311,12 @@ spotify_init(void)
       goto thread_fail;
     }
 
+#if defined(HAVE_PTHREAD_SETNAME_NP)
+  pthread_setname_np(tid_spotify, "spotify");
+#elif defined(HAVE_PTHREAD_SET_NAME_NP)
+  pthread_set_name_np(tid_spotify, "spotify");
+#endif
+
   DPRINTF(E_DBG, L_SPOTIFY, "Spotify init complete\n");
   return 0;
 
